/*
 * app.c
 *
 * Main event loop etc. for the Red Feline Backup application
 *
 * Copyright (C) 2004 Daniel Thompson <d.thompson@gmx.net> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include "rftidy.h"

static UInt16 AppNumRecords(const char *dbName)
{
	LocalID id;
	DmOpenRef db;
	UInt16 res;
	
	id = DmFindDatabase(0, dbName);
	if (id == NULL) {
		return 0;
	}
	
	db = DmOpenDatabase(0, id, dmModeReadWrite);
	if (NULL == db) {
		return 0;
	}
	
	res = DmNumRecordsInCategory(db, dmAllCategories);
	
	DmCloseDatabase(db);
	return res;
}

static void AppUpdateStatisticsLabel(FormPtr form, UInt16 labelId, const char *dbName)
{
	UInt16 n;
	Char s[maxStrIToALen];
	n = AppNumRecords(dbName);
	StrIToA(s, n);
	FrmCopyLabel(form, labelId, s);
}

static Err AppStart(void)
{
	/* TODO: check version of PalmOS (>= ???, < OS6) */
	
	FrmGotoForm(kFormMain);
	return errNone;
}

static void AppStop(void)
{
	/* TODO: manage preferences if required */
}

static Boolean AppHandleEvent(EventPtr event)
{
	Boolean eventHandled = true;
	Int formID;
	FormPtr form;
	UInt16 buttonID;
	MemHandle versionHandle;
	Char* versionStr;

	switch (event->eType) {
	case frmLoadEvent:
		formID = event->data.frmLoad.formID;
		form = FrmInitForm(formID);
		
		FrmSetActiveForm(form);

		/* choose a form handler based on the form ID */
		switch (formID) {
		case kFormMain:
			FrmSetEventHandler(form, MainFormHandleEvent);
			break;
		case kFormProgress:
			FrmSetEventHandler(form, ProgressFormHandleEvent);
			break;
		default:
			ErrFatalDisplay("Invalid Form Load Event");
			break;
		}
		
		break;
		
	case menuEvent:
		/* handle menu events that have the same action for all forms */
		switch (event->data.menu.itemID) {
		case kMenuItemAbout:
			form = FrmInitForm(kFormAbout);
			
			/* update the version string (be sure that the old label is longer than the new one) */
			versionHandle = DmGetResource('tver', 1);
			versionStr = (Char *) MemHandleLock(versionHandle);
			FrmCopyLabel(form, kLabelVersion, versionStr);
			MemHandleUnlock(versionHandle);
			DmReleaseResource(versionHandle);
						
			buttonID = FrmDoDialog(form);
			FrmDeleteForm(form);
			break;
				
		case kMenuItemStatistics:
			form = FrmInitForm(kFormStatistics);
			AppUpdateStatisticsLabel(form, kLabelAddressBookRecords, "AddressDB");
			AppUpdateStatisticsLabel(form, kLabelDataBookRecords, "DatebookDB");
			
			buttonID = FrmDoDialog(form);
			FrmDeleteForm(form); 
			break;
			
		default:
			eventHandled = false;
			break;
		}
		break;

	default:
		eventHandled = false;
		break;
	}

	return eventHandled;
}

Boolean AppHandleEventsUntil(eventsEnum eType)
{
	EventType event;
	UInt16 err;
	Boolean eventHandled;
	
	/* this is our main event loop */
	do {
		EvtGetEvent(&event, evtWaitForever);

		/* ANSI C mandates lazy evaluation! */
		eventHandled = SysHandleEvent(&event) ||
		               MenuHandleEvent(0, &event, &err) ||
		               AppHandleEvent(&event) ||
		               FrmDispatchEvent(&event);
		               
	} while (eType != event.eType && appStopEvent != event.eType);
	
	if (eType == event.eType) {
		return true;
	}
	
	/* this is an appStopEvent during an inner event loop, we requeue and drop upwards */
	EvtAddEventToQueue(&event);
	return false;
}

DWord PilotMain (UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
	Err err;
	
	if (cmd == sysAppLaunchCmdNormalLaunch) {
		err = AppStart();
		if (errNone == err) {
			AppHandleEventsUntil(appStopEvent);
			AppStop();
			FrmCloseAllForms();
		}
	}

	return errNone;
}